function [u,v,p] = StokesDGS(u,v,p,nu,f1,f2,itstep,g)
% Copyright (C)  Long Chen. See COPYRIGHT.txt for details.

N = size(f1,1);
h = 1/N;

if nargin<8
    g = zeros(N,N);
end
if u == 0
    u = zeros(N,N+1);
end
if v == 0
    v = zeros(N+1,N);
end
if p == 0
    p = zeros(N,N);
end

for k = 1:itstep % Distributive Gauss-Seidel relaxation
    
[u,v] = updatevelocity(u,v,p,nu,f1,f2);

% DGS relaxation
% interior pressure points
for i = 2:N-1
    for j = 2:N-1
        % compute residual
        rc = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
%         p(i,j) = p(i,j) + rc;
        delta = rc*h/4;
        % update u
        u(i,j+1) = u(i,j+1) + delta;
        u(i,j) = u(i,j) - delta;
        % update v
        v(i,j) = v(i,j) + delta;
        v(i+1,j) = v(i+1,j) - delta;
        % update p
        p(i,j) = p(i,j) + rc;
        p(i,j-1) = p(i,j-1) - rc/4;
        p(i,j+1) = p(i,j+1) - rc/4;
        p(i+1,j) = p(i+1,j) - rc/4;
        p(i-1,j) = p(i-1,j) - rc/4;
    end
end
% boundary nodes
i = 1;
for j = 2:N-1
    % compute residual
    rc = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
%         p(i,j) = p(i,j) + rc;
    delta = rc*h/3;
    % update u
    u(i,j+1) = u(i,j+1) + delta;
    u(i,j) = u(i,j) - delta;
    % update v
    v(i+1,j) = v(i+1,j) - delta;
    % update p
    p(i,j) = p(i,j) + rc;
    p(i,j-1) = p(i,j-1) - rc/3;
    p(i,j+1) = p(i,j+1) - rc/3;
    p(i+1,j) = p(i+1,j) - rc/3;
end
i = N;
for j = 2:N-1
    % compute residual
    rc = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
%         p(i,j) = p(i,j) + rc;
    delta = rc*h/3;
    % update u
    u(i,j+1) = u(i,j+1) + delta;
    u(i,j) = u(i,j) - delta;
    % update v
    v(i,j) = v(i,j) + delta;
    % update p
    p(i,j) = p(i,j) + rc;
    p(i,j-1) = p(i,j-1) - rc/3;
    p(i,j+1) = p(i,j+1) - rc/3;
    p(i-1,j) = p(i-1,j) - rc/3;
end

j = 1;
for i = 2:N-1
    % compute residual
    rc = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
    delta = rc*h/3;
    % update u
    u(i,j+1) = u(i,j+1) + delta;
    % update v
    v(i,j) = v(i,j) + delta;
    v(i+1,j) = v(i+1,j) - delta;
    % update p
    p(i,j) = p(i,j) + rc;
    p(i-1,j) = p(i-1,j) - rc/3;
    p(i+1,j) = p(i+1,j) - rc/3;
    p(i,j+1) = p(i,j+1) - rc/3;
end

j = N;
for i = 2:N-1
    % compute residual
    rc = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
%         p(i,j) = p(i,j) + rc;
    delta = rc*h/3;
    % update u
    u(i,j) = u(i,j) - delta;
    % update v
    v(i,j) = v(i,j) + delta;
    v(i+1,j) = v(i+1,j) - delta;
    % update p
    p(i,j) = p(i,j) + rc;
    p(i-1,j) = p(i-1,j) - rc/3;
    p(i+1,j) = p(i+1,j) - rc/3;
    p(i,j-1) = p(i,j-1) - rc/3;
end

% update corners
i = 1; j = 1;
% compute residual
rc = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
delta = rc*h/2;
% update u
u(i,j+1) = u(i,j+1) + delta;
% update v
v(i+1,j) = v(i+1,j) - delta;
% update p
p(i,j) = p(i,j) + rc;
p(i,j+1) = p(i,j+1) - rc/2;
p(i+1,j) = p(i+1,j) - rc/2;

i = 1; j = N;
% compute residual
rc = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
delta = rc*h/2;
% update u
u(i,j) = u(i,j) - delta;
% update v
v(i+1,j) = v(i+1,j) - delta;
% update p
p(i,j) = p(i,j) + rc;
p(i,j-1) = p(i,j-1) - rc/2;
p(i+1,j) = p(i+1,j) - rc/2;

i = N; j = 1;
% compute residual
rc = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
delta = rc*h/2;
% update u
u(i,j+1) = u(i,j+1) + delta;
% update v
v(i,j) = v(i,j) + delta;
% update p
p(i,j) = p(i,j) + rc;
p(i,j+1) = p(i,j+1) - rc/2;
p(i-1,j) = p(i-1,j) - rc/2;

i = N; j = N;
% compute residual
rc = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
delta = rc*h/2;
% update u
u(i,j) = u(i,j) - delta;
% update v
v(i,j) = v(i,j) + delta;
% update p
p(i,j) = p(i,j) + rc;
p(i,j-1) = p(i,j-1) - rc/2;
p(i-1,j) = p(i-1,j) - rc/2;

end