function [u,v,p] = StokesVcycle(level,smoothingstep,pde)
% Copyright (C)  Long Chen. See COPYRIGHT.txt for details.

%% Initialization
N = 2^level;
p = zeros(N,N);
ru = cell(level,1);            % record residual in each level
rv = cell(level,1);
rp = cell(level,1);
eu = cell(level,1);            % record err in each level
ev = cell(level,1);
ep = cell(level,1);
g = zeros(N,N);    % used for residual of equation div e = g 

%% PDE information
nu = 1;
if isfield(pde,'nu'), nu = pde.nu; end
[fu,fv] = evaluaterhs(pde.fu,pde.fv,N);
[fu,fv,u,v] = evaluateDirichlet(pde.gDu,pde.gDv,fu,fv);

%% DGS-MG iteration
k = 1; maxIt = 100; err = 1; tol = 0.1/N^2;
while (err > tol) && (k <= maxIt)
    % form residual
    [ru{level},rv{level},rp{level}] = formresidual(fu,fv,g,u,v,p,nu);
    for i = level:-1:3
        % presmoothing
        [eu{i},ev{i},ep{i}] = StokesDGS(0,0,0,nu,ru{i},rv{i},smoothingstep,rp{i});
        [tempru,temprv,temprp] = formresidual(ru{i},rv{i},rp{i},eu{i},ev{i},ep{i},nu);        
        [ru{i-1},rv{i-1},rp{i-1}] = restriction(tempru,temprv,temprp);
    end
%     [eu{2},ev{2},ep{2}] = StokesDGSUzawa(0,0,0,nu,ru{2},rv{2},100,rp{2});
    [eu{2},ev{2},ep{2}] = StokesDGS(0,0,0,nu,ru{2},rv{2},100,rp{2});
    for i = 3:level
        [tempu,tempv,tempp] = prolongation(eu{i-1},ev{i-1},ep{i-1});
        eu{i} = eu{i} + tempu; 
        ev{i} = ev{i} + tempv;
        ep{i} = ep{i} + tempp;
        % post-smoothing
        [eu{i},ev{i},ep{i}] = StokesDGS(eu{i},ev{i},ep{i},nu,ru{i},rv{i},smoothingstep,rp{i});        
    end
    u = u + eu{level};
    v = v + ev{level};
    p = p + ep{level};
    err = max([max(abs(eu{level})),max(abs(ev{level})),max(abs(ep{level}))]);
    fprintf('iteration = %2.0u, err = %12.8g \n', k, err);
    k = k + 1;
end