function [ru,rv,rp] = formresidual(f1,f2,g,u,v,p,nu)
% Copyright (C)  Long Chen. See COPYRIGHT.txt for details.

N = size(p,1);
h = 1/N;
ru = zeros(N,N+1);
rv = zeros(N+1,N);
rp = zeros(N,N);
% Momentum equation for u
% nu(4u(i,j) - u(i-1,j) - u(i+1,j) - u(i,j-1) - u(i,j+1))/h^2 
%  + (p(i,j) - p(i-1,j))/h = f1(i,j)
% interiori nodes
for j = 2:N % full index 1:N+1; interior index 2:N
    for i = 2:N-1 % full index 1:N; interior index 2:N-1
        ru(i,j) = f1(i,j)- (p(i,j) - p(i,j-1))/h ...
               - nu*(4*u(i,j) - u(i-1,j) - u(i+1,j) - u(i,j-1) - u(i,j+1))/h^2;
    end
end

% Momentum equation for v
% nu(4v(i,j) - v(i-1,j) - v(i+1,j) - v(i,j-1) - v(i,j+1))/h^2 
%  + (p(i,j) - p(i,j-1))/h = f2(i,j)
% interiori nodes
for j = 2:N-1 % full index 1:N; interior index 2:N-1
    for i = 2:N % full index 1:N+1; interior index 2:N
        rv(i,j) = f2(i,j) - (p(i-1,j) - p(i,j))/h ...
              - nu*(4*v(i,j) - v(i-1,j) - v(i+1,j) - v(i,j-1) - v(i,j+1))/h^2;
    end
end

% boundary nodes for u
% nu(5u(i,j) - u(i-1,j) - u(i+1,j) - u(i,j+1))/h^2
%  + (p(i,j) - p(i-1,j))/h = f1(i,j)
i = 1;
for j = 2:N
    ru(i,j) = f1(i,j) - (p(i,j) - p(i,j-1))/h ... 
             - nu*(5*u(i,j) - u(i+1,j) - u(i,j-1) - u(i,j+1))/h^2;
end
i = N;
for j = 2:N
    ru(i,j) = f1(i,j) - (p(i,j) - p(i,j-1))/h ... 
             - nu*(5*u(i,j) - u(i-1,j) - u(i,j-1) - u(i,j+1))/h^2;
end

% boundary nodes for v
% nu(5v(i,j) - v(i+1,j) - v(i,j-1) - v(i,j+1))/h^2 
%  + (p(i,j) - p(i,j-1))/h = f2(i,j)
j = 1;
for i = 2:N
    rv(i,j) = f2(i,j) - (p(i-1,j) - p(i,j))/h ...
            - nu*(5*v(i,j) - v(i-1,j) - v(i+1,j) - v(i,j+1))/h^2;
end
j = N;
for i = 2:N
    rv(i,j) = f2(i,j) - (p(i-1,j) - p(i,j))/h ...
            - nu*(5*v(i,j) - v(i-1,j) - v(i+1,j) - v(i,j-1))/h^2;
end

% pressure
for i = 1:N
    for j = 1:N
        % compute residual
        rp(i,j) = g(i,j) - (u(i,j+1) - u(i,j))/h - (v(i,j) - v(i+1,j))/h;
    end
end
