function [u,v] = updatevelocity(u,v,p,nu,f1,f2)
% Copyright (C)  Long Chen. See COPYRIGHT.txt for details.

N = size(p,1);
h = 1/N;

% Momentum equation for u
% nu(4u(i,j) - u(i-1,j) - u(i+1,j) - u(i,j-1) - u(i,j+1))/h^2 
%  + (p(i,j) - p(i-1,j))/h = f1(i,j)
% interiori nodes
for j = 2:N % full index 1:N+1; interior index 2:N
    for i = 2:N-1 % full index 1:N; interior index 2:N-1
        u(i,j) = (h^2*f1(i,j)+u(i-1,j)+u(i+1,j)+u(i,j-1)+u(i,j+1)...
               - h*(p(i,j) - p(i,j-1)))/(4*nu);
    end
end
% boundary condition
% u =0 at x=0, x= 1 and du/dn = g1 at y=0, y=1
i = 1;
% nu(5u(i,j) - u(i-1,j) - u(i+1,j) - u(i,j+1))/h^2
%  + (p(i,j) - p(i-1,j))/h = f1(i,j)
for j = 2:N
        u(i,j) = (h^2*f1(i,j)+u(i+1,j)+u(i,j-1)+u(i,j+1) ...
               - h*(p(i,j) - p(i,j-1)))/(5*nu);
end
i = N;
% nu(5u(i,j) - u(i-1,j) - u(i+1,j) - u(i,j+1))/h^2
%  + (p(i,j) - p(i-1,j))/h = f1(i,j)
for j = 2:N
        u(i,j) = (h^2*f1(i,j)+u(i-1,j)+u(i,j-1)+u(i,j+1) ...
               - h*(p(i,j) - p(i,j-1)))/(5*nu);
end

% Momentum equation for v
% nu(4v(i,j) - v(i-1,j) - v(i+1,j) - v(i,j-1) - v(i,j+1))/h^2 
%  + (p(i,j) - p(i,j-1))/h = f2(i,j)
% interiori nodes
for j = 2:N-1 % full index 1:N; interior index 2:N-1
    for i = 2:N % full index 1:N+1; interior index 2:N
        v(i,j) = (h^2*f2(i,j)+v(i-1,j)+v(i+1,j)+v(i,j-1)+v(i,j+1)...
               - h*(p(i-1,j) - p(i,j)))/(4*nu);
    end
end
% boundary condition
% v =0 at y=0, y= 1 and dv/dn = g2 at x=0, x=1
% nu(5v(i,j) - v(i+1,j) - v(i,j-1) - v(i,j+1))/h^2 
%  + (p(i,j) - p(i,j-1))/h = f2(i,j)
j = 1;
for i = 2:N
        v(i,j) = (h^2*f2(i,j) +v(i-1,j)+v(i+1,j)+v(i,j+1)...
               - h*(p(i-1,j) - p(i,j)))/(5*nu);
end
j = N;
% nu(5v(i,j) - v(i-1,j) - v(i+1,j) - v(i,j+1))/h^2
%  + (p(i,j) - p(i,j-1))/h = f2(i,j)
for i = 2:N
        v(i,j) = (h^2*f2(i,j)+v(i-1,j)+v(i+1,j)+v(i,j-1)...
               - h*(p(i-1,j) - p(i,j)))/(5*nu);
end